<?php
/**
 * Contenido extension pack (mpContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Cache
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2009-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


defined('CON_FRAMEWORK') or die('Illegal call');


/**
 * Bootstrap class. Initializes Contenido backend.
 *
 *
 * @TODO: create a abstact bootstrap, see ConMp_Plugin_Bootstrap_Abstract
 *
 * @author      Murat Purc <murat@purc.de>
 * @copyright 	2009 Murat Purc
 * @category    ContenidoX
 * @package     Bootstrap
 */
class ConMp_Bootstrap
{
    /**
     * Self instance
     * @var  ConMp_Bootstrap
     */
    private static $_instance;

    /**
     * Flag to store loaded status. Will be set to true in run() function.
     * @var  bool
     */
    private static $_run = false;


    /**
     * Prevent public instantiation
     */
    protected function __construct()
    {
        // depends on mpContenidoX, load it before...
        plugin_include('mpContenidoX', 'includes/config.plugin.php');
    }


    /**
     * Disable cloning
     */
    private function __clone()
    {
    }


    /**
     * Returns self instance
     *
     * @return  ConMp_Bootstrap
     */
    public static function getInstance()
    {
        if (self::$_instance == null) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }


    /**
     * Main function to call. Invokes all boostrap related functions.
     *
     * @retun void
     */
    public function run()
    {
        if (self::$_run) {
            return;
        }
        self::$_run = true;

        $methods = get_class_methods($this);
        foreach ($methods as $method) {
            if (5 < strlen($method) && '_init' === substr($method, 0, 5)) {
                #echo "$method<br />";
                $this->$method();
            }
        }
    }


    /**
     * Sets some constants.
     *
     * @retun void
     */
    protected function _initConstants()
    {
        // path to contenido installation (e. g. /var/www/, C:/htdocs/)
        define('CONTENIDOX_INSTALLATION_PATH', str_replace('\\', '/', realpath($GLOBALS['cfg']['path']['contenido'] . '..')) . '/');

        // path to contenido backend (e. g. /var/www/contenido/, C:/htdocs/contenido/)
        define('CONTENIDOX_PATH', $GLOBALS['cfg']['path']['contenido']);

        // path to ConMp library (e. g. /var/www/contenido/plugins/mpContenidoX/libs/, C:/htdocs/contenido/plugins/mpContenidoX/libs/)
        define('CONMP_LIB_PATH', PLUGIN_MPCONTENIDOX_PATH . 'libs/');
    }


    /**
     * Includes some needed sources.
     *
     * @retun void
     */
    protected function _initIncludes()
    {
        require_once(CONMP_APPLICATION_PATH . 'functions/conX.php');
    }


    /**
     * Loads the main configuration from configs folder (e. g. contenido/plugins/mpContenidoX/application/configs/config.php).
     *
     * @retun void
     */
    protected function _initConfigurations()
    {
//        require_once(PLUGIN_MPCONTENIDOX_PATH . 'ConMp/Config.php');
        try {
            ConMp_Config::getInstance()->loadConfig(CONMP_APPLICATION_PATH . 'configs/', 'config');
        } catch (Exception $e) {
            throw new Exception('Missing configuration file configs/config.php. ' . $e->getMessage());
        }
    }


    /**
     * Adds the autoloader.
     *
     * @retun void
     */
/*
    protected function _initAutoload()
    {
        spl_autoload_register(array('ConMp_Bootstrap', 'autoload'));
    }
*/

    /**
     * Fills registry with common applicationwide needed objects.
     *
     * @retun void
     */
    protected function _initRegistry()
    {
        $request = ConMp_Request::getInstance();
        ConMp_Registry::set('oRequest', $request);
        ConMp_Registry::set('oDB', new DB_Contenido());
        ConMp_Registry::set('oSess', $GLOBALS['sess']);
        ConMp_Registry::set('oAuth', $GLOBALS['auth']);
        ConMp_Registry::set('oPerm', $GLOBALS['perm']);
        ConMp_Registry::set('oUser', $GLOBALS['currentuser']);
        ConMp_Registry::set('oCecRegistry', $GLOBALS['_cecRegistry']);
    }


    /**
     * The autoload implementation of ConMp.
     *
     * @param  string  $className  The classname to load
     * @retun void
     */
/*
    public static function autoload($className)
    {
        $file = '';
        if ($className == 'ConMp' || (strlen($className) > 11 && substr($className, 0, 11) == 'ConMp')) {
            $file = CONMP_LIB_PATH . str_replace('_', '/', $className) . '.php';
        }

        if (empty($file)) {
            return;
        }

        if (is_file($file)) {
            require_once($file);
        } else {
            throw new Exception('ConMp_Bootstrap::autoload(): Couldn\'t load class file ' . $className);
        }
    }
*/

}