<?php
/**
 * Contenido extension pack (mpContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Controller
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2009-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


defined('CON_FRAMEWORK') or die('Illegal call');


cInclude('includes', 'functions.str.php');
cInclude('includes', 'functions.tpl.php');
cInclude('includes', 'functions.lang.php');

cInclude('classes', 'class.htmlelements.php');
cInclude('classes', 'class.ui.php');
cInclude('classes', 'Debug/Debug_Visible.class.php');


/**
 * Strcorrect controller.
 *
 * Tool to rebuild defected category structure on a Contenido 4.8.x installation.
 * Based on contenido/includes/include.con_str_overview.php.
 *
 * @TODO
 * The controller has too much code and business logic. It should be stripped
 * down to its real mission and the business logic should be outsourced into a
 * model...
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Controller
 * @author      Murat Purc <murat@purc.de>
 */
class StrcorrectController extends ConMp_Controller_Backend_Abstract
{

    /**
     * (non-PHPdoc)
     * @see contenido/plugins/mpContenidoX/libs/ConMp/Controller/Backend/ConMp_Controller_Backend_Abstract#init()
     */
    public function init()
    {
        // debug message
        $this->_debugMsg = '';

        // category level cache list
        $this->_catLevelsCache = array();

        // navigaton tree
        $this->_navigationTreeLoaded = false;
        $this->_navigationTree = array();

        // category list
        $this->_wholeList = array();

        // some default templates strings
        $this->_changeClientOptions   = '<option>-- Change client --</option>';
        $this->_changeLanguageOptions = '<option>-- Change language --</option>';

        // check rights flag
        $this->_checkGlobalRights = false;

        // database instances
        $this->_db  = ConMp_Registry::get('oDB');
        $this->_db2 = new DB_Contenido();
        $this->_db3 = new DB_Contenido();

        // Client and language initialization
        $this->_clientName = '';
        $this->_languageName = '';
        $this->_initClientAndLanguage();
    }


    /**
     * Index action. Loads and displays the nav tree.
     *
     * @return  void
     */
    public function indexAction()
    {
        $this->_loadNavigationTree();

        if (count($this->_navigationTree[0])) {
            $sCategories = $this->_processStrTree(0, 0, 0);
        } else {
            $sCategories = 'no cats';
        }

        $tpl = ConMp_Template_Extended::getInstance($this->_templateFile, 'LAYOUT_MAIN');

        $tpl->set('s', 'CHANGE_LANGUAGE_OPTIONS', $this->_changeLanguageOptions);
        $tpl->set('s', 'CHANGE_CLIENT_OPTIONS', $this->_changeClientOptions);
        $tpl->set('s', 'DEBUG_ATTRIBUTES', (($this->_cfg->mpContenidoXApp['contentDebug']) ? ' checked="checked"' : ''));
        $tpl->set('s', 'DEBUG', (int) $this->_cfg->mpContenidoXApp['contentDebug']);
        $tpl->set('s', 'LANG', $this->_lang);
        $tpl->set('s', 'CLIENT', $this->_client);
        $tpl->set('s', 'COLLAPSED_STATUS', trim(strip_tags($this->_request->get('collapsed_status', ''))));
        $tpl->set('s', 'DEBUG_MSG', conX_debugGetAll());

        $user = new User();
        $username = $user->getRealname($this->_auth->auth['uid']);

        $tpl->set('s', 'PAGE', CONTENIDOX_PAGE);
        $tpl->set('s', 'SID', $this->_sess->id);
        $tpl->set('s', 'USERNAME', $username);
        $tpl->set('s', 'CATS', $sCategories);
        $tpl->set('s', 'AREA', $this->_area);
        $tpl->set('s', 'ACTION', $this->_action);
        $tpl->set('s', 'SESSION', $this->_contenido);
        $tpl->set('s', 'DIRECTION', ' dir="' . langGetTextDirection($this->_lang) . '"');
        $tpl->set('s', 'AJAXURL', $this->_cfg->path['contenido_fullhtml'].'ajaxmain.php');

        $tpl->generate();
    }


    /**
     * Performs the correction on navigation item and invokes indexAction
     *
     * @return  void
     */
    public function correctstrAction()
    {
        $params = $this->_request->getParams();
        $db = ConMp_Registry::get('oDB');

        // correct parentid
        if (isset($params['parentid_old']) && isset($params['parentid_new']) && isset($params['correct_cat'])) {
            $oldParentId = (int) $params['parentid_old'];
            $newParentId = (int) $params['parentid_new'];
            $correctCat  = (int) $params['correct_cat'];
            if ($oldParentId !== $newParentId) {
                strMoveSubtree($correctCat, $newParentId);
            }
        }

        // correct preid
        if (isset($params['preid_old']) && isset($params['preid_new']) && isset($params['correct_cat'])) {
            $oldPreId   = (int) $params['preid_old'];
            $newPreId   = (int) $params['preid_new'];
            $correctCat = (int) $params['correct_cat'];

            if ($oldPreId !== $newPreId) {
                $sql = 'UPDATE %s SET preid=%d WHERE preid=%d AND idcat=%d AND idclient=%d';
                $sql = sprintf($sql, $this->_cfg->tab['cat'], $newPreId, $oldPreId, $correctCat, $this->_client);
                echo "<pre>$sql</pre>";
                $this->_db->query($sql);
            }
        }

        // correct post
        if (isset($params['postid_old']) && isset($params['postid_new']) && isset($params['correct_cat'])) {
            $oldPostId  = (int) $params['postid_old'];
            $newPostId  = (int) $params['postid_new'];
            $correctCat = (int) $params['correct_cat'];

            if ($oldPostId !== $newPostId) {
                $sql = 'UPDATE %s SET postid=%d WHERE postid=%d AND idcat=%d AND idclient=%d';
                $sql = sprintf($sql, $this->_cfg->tab['cat'], $newPostId, $oldPostId, $correctCat, $this->_client);
                echo "<pre>$sql</pre>";
                $this->_db->query($sql);
            }
        }

        // correct level
        if (isset($params['level_old']) && isset($params['level_new']) && isset($params['correct_cat'])) {
            $oldLevel   = (int) $params['level_old'];
            $newLevel   = (int) $params['level_new'];
            $correctCat = (int) $params['correct_cat'];

            if ($oldLevel !== $newLevel) {
                $sql = 'SELECT idtree FROM %s WHERE idcat=%d';
                $sql = sprintf($sql, $this->_cfg->tab['cat_tree'], $correctCat);
                echo "<pre>$sql</pre>";
                $this->_db->query($sql);
                if ($this->_db->next_record()) {
                    $sql = 'UPDATE %s SET level=%d WHERE level=%d AND idcat=%d';
                    $sql = sprintf($sql, $this->_cfg->tab['cat_tree'], $newLevel, $oldLevel, $correctCat);
                } else {
                    $nextId = $this->_db->nextid($this->_cfg->tab['cat_tree']);
                    $sql = 'INSERT INTO %s VALUES (%d, %d, %d)';
                    $sql = sprintf($sql, $this->_cfg->tab['cat_tree'], $nextId, $correctCat, $newLevel);
                }
                echo "<pre>$sql</pre>";
                $this->_db->query($sql);
            }
        }

        $this->indexAction();
    }


    /**
     * Proxy for strRemakeTreeTable()
     *
     * @return  void
     */
    public function remakecattreeAction()
    {
        strRemakeTreeTable();
        $this->indexAction();
    }


    /**
     * Displays the tree structure as a table
     *
     * @return  void
     */
    public function tableAction()
    {
        $this->_loadNavigationTree();

        $tpl = ConMp_Template_Extended::getInstance($this->_templateFile, 'LAYOUT_TABLE');

        $tpl->set('s', 'CLIENT_NAME', $this->_clientName);
        $tpl->set('s', 'LANGUAGE_NAME', $this->_languageName);

        if (count($this->_navigationTree[0])) {
            $this->_renderNavigationTreeItem(0, $tpl);
        } else {
            $tpl->set('d', 'IDCAT', '-');
            $tpl->set('d', 'NAMEATTRIB', '');
            $tpl->set('d', 'NAME', 'No structure data');
            $tpl->set('d', 'LEVEL', '');
            $tpl->set('d', 'PARENTID', '');
            $tpl->set('d', 'PREID', '');
            $tpl->set('d', 'POSTID', '');
            $tpl->next();
        }

        $tpl->generate();
    }


    /**
     * Loads the navigation tree structure.
     *
     * @return  void
     */
    protected function _loadNavigationTree()
    {
        if ($this->_navigationTreeLoaded) {
            return;
        }

        $sql = "SELECT
                    a.preid, a.postid, a.parentid, c.idcat, c.level, b.name, b.public,
                    b.visible, d.idtpl, b.idlang
                FROM
                    (".$this->_cfg->tab['cat']." AS a,
                    ".$this->_cfg->tab['cat_lang']." AS b,
                    ".$this->_cfg->tab['cat_tree']." AS c)
                LEFT JOIN
                    ".$this->_cfg->tab['tpl_conf']." AS d
                    ON d.idtplcfg = b.idtplcfg
                WHERE
                    a.idclient  = '". $this->_client ."' AND
                    b.idlang    = '". $this->_lang ."' AND
                    c.idcat     = b.idcat AND
                    b.idcat     = a.idcat
                ORDER BY
                    c.idtree ASC";
//##//        conX_debugAdd($sql, 'structure sql');
        $this->_db->query($sql);
        $aIn = array();
        while ($this->_db->next_record()) {
            $aIn[] = (int) $this->_db->f('idcat');
        }
        $sIn = implode(',', $aIn);

        $sql2 = "SELECT b.idcat, a.idart, idlang
                 FROM ".$this->_cfg->tab['art_lang']." AS a, ".$this->_cfg->tab['cat_art']." AS b
                 WHERE b.idcat IN (". $sIn.") AND a.idlang = '". $this->_lang ."'
                 AND b.idart = a.idart";
        $this->_db->query($sql2);
//##//        conX_debugAdd($sql2, 'articles sql');

        $aArtCache = array();

        while ($this->_db->next_record()) {
            $aArtCache[$this->_db->f('idcat')][$this->_db->f('idart')][$this->_db->f('idlang')] = 'x';
        }

        $this->_db->query($sql);

        while ($this->_db->next_record()) {
            $entry = array();
            $entry['articles'] = false;
            if ($this->_db->f('idlang') == $this->_lang) {
                $arts = array();
                if (isset($aArtCache[$this->_db->f('idcat')])) {
                    foreach ($aArtCache[$this->_db->f('idcat')] as $key => $value) {
                        foreach ($value as $key2 => $value2) {
                            $arts[$key][$key2] = 1;
                        }
                    }
                }
                foreach ($arts as $idart => $entry) {
                    if (is_array($entry)) {
                        if (!array_key_exists($this->_lang, $entry)) {
                            //$entry['articles'] = true;
                            $aIsArticles[$this->_db->f('idcat')] = true;
                            break;
                        }
                    }
                }
            }
        }

        // Check global rights
        $this->_checkGlobalRights = $this->_perm->have_perm_area_action('con', 'con_makestart');
        if (!$this->_checkGlobalRights) {$this->_checkGlobalRights = $this->_perm->have_perm_area_action('con', 'con_deleteart');}
        if (!$this->_checkGlobalRights) {$this->_checkGlobalRights = $this->_perm->have_perm_area_action('con', 'con_makeonline');}
        if (!$this->_checkGlobalRights) {$this->_checkGlobalRights = $this->_perm->have_perm_area_action('con', 'con_tplcfg_edit');}
        if (!$this->_checkGlobalRights) {$this->_checkGlobalRights = $this->_perm->have_perm_area_action('con', 'con_makecatonline');}
        if (!$this->_checkGlobalRights) {$this->_checkGlobalRights = $this->_perm->have_perm_area_action('con', 'con_changetemplate');}

        $sql = "SELECT DISTINCT
                    a.idcat, a.parentid, a.preid, a.postid, a.parentid, b.name, b.idlang,
                    b.visible, b.public, c.level, d.idtpl
                FROM {$this->_cfg->tab['cat']} AS a
                LEFT JOIN {$this->_cfg->tab['cat_lang']} AS b ON a.idcat = b.idcat
                LEFT JOIN {$this->_cfg->tab['cat_tree']} AS c ON (a.idcat = c.idcat AND b.idcat = c.idcat)
                LEFT JOIN {$this->_cfg->tab["tpl_conf"]} AS d ON b.idtplcfg = d.idtplcfg
                WHERE  a.idclient = {$this->_client} ORDER BY b.idlang DESC, c.idtree ASC ";
        $this->_db->query($sql);
//##//        conX_debugAdd($sql, 'structure sql');

        $currentuser = ConMp_Registry::get('oUser');

        $sExpandList = $currentuser->getUserProperty('system', 'con_cat_expandstate');
        if ($sExpandList != '') {
            $conexpandedList = unserialize($currentuser->getUserProperty('system', 'con_cat_expandstate'));
        }
        if (!is_array($conexpandedList)) {
            $conexpandedList = array();
        }

        while ($this->_db->next_record()) {
            if (!isset ($this->_navigationTree[$this->_db->f('parentid')][$this->_db->f('idcat')]) && $this->_db->f('idlang') == $this->_lang) {
                $rs = $this->_db->Record;
                $rs['collapsed']    = (in_array($rs['idcat'], $conexpandedList)) ? false : true;
                $rs['forcedisplay'] = ($this->_perm->have_perm_item('con', $rs['idcat'])) ? 1 : 0;
                $rs['active']       = ($idcat == $rs['idcat']) ? true : false;
                $rs['islast']       = false;
                $rs['articles']     = $aIsArticles[$rs['idcat']];

                $this->_navigationTree[$rs['parentid']][$rs['idcat']] = $rs;
            }
        }

        $this->_navigationTreeLoaded = true;
    }


    /**
     * Processes the rendering of category tree structure
     *
     * @param   int    $iIdcat
     * @param   int    $level
     * @param   int    $prevLevel
     * @return  void
     */
    protected function _processStrTree($iIdcat, $level, $prevLevel)
    {
        $tpl = ConMp_Template_Extended::getNewInstance($this->_templateFile, 'PARTIAL_CATEGORY_LIST');
        $tpl->reset();

        $tplCat = ConMp_Template_Extended::getInstance($this->_templateFile, 'PARTIAL_CAT_ITEM');

        $iIdcat   = (int) $iIdcat;
        $preIdCat = $iIdcat;

        // save current category level
        $this->_addToLevelCache($iIdcat, $level);

        foreach ($this->_navigationTree[$iIdcat] as $sKey => $aValue) {

            $cfgdata     = '';
            $aCssClasses = array();

            // check rights per cat
            $bCheckRights = (!$this->_checkGlobalRights) ? false : true;
            if (!$bCheckRights) {
                $bCheckRights = ($aValue['forcedisplay'] == 1) ? true : false;
            }

            $currCatId = (int) $aValue['idcat'];
            $name  = $aValue['name'];

            if ($bCheckRights) {

                $idtpl = ((int) $aValue['idtpl'] > 0) ? (int) $aValue['idtpl'] : 0;

                // save current category level
                $this->_addToLevelCache($currCatId, $aValue['level']);

                // select the appropriate folder- image depending on the structure properties
                if ($this->_cfg->is_start_compatible == true) {
                    $sql2 = "SELECT
                                c.is_start AS is_start, a.online AS online, a.idlang AS idlang
                            FROM
                                ".$this->_cfg->tab['art_lang']." AS a,
                                ".$this->_cfg->tab['art']." AS b,
                                ".$this->_cfg->tab['cat_art']." AS c
                            WHERE
                                a.idart = b.idart AND
                                b.idclient = '". $this->_client ."' AND
                                b.idart = c.idart AND
                                c.idcat = '". $currCatId ."'";
                } else {
                    $sql2 = "SELECT
                                a.idartlang AS idartlang, a.online AS online, a.idlang AS idlang
                            FROM
                                ".$this->_cfg->tab['art_lang']." AS a,
                                ".$this->_cfg->tab['art']." AS b,
                                ".$this->_cfg->tab['cat_art']." AS c
                            WHERE
                                a.idart = b.idart AND
                                b.idclient = '". $this->_client ."' AND
                                b.idart = c.idart AND
                                c.idcat = '". $currCatId ."'";
                }

                $this->_db2->query($sql2);

                $noStart   = true;
                $noOnline  = true;
                while ($this->_db2->next_record()) {
                    // start article check
                    if ($this->_cfg->is_start_compatible == true) {
                        $noStart = ($this->_db2->f('is_start') != 1);
                    } else {
                        $noStart = !isStartArticle($this->_db2->f('idartlang'), $currCatId, $this->_lang, $this->_db3);
                    }
                    // online check
                    $noOnline = ($this->_db2->f('online') != 1);
                    if (!$noStart && !$noOnline) {
                        // Exit loop if both vars are already false
                        break;
                    }
                }

                if ($aValue['online'] == 1) {
                    // category is online
                    if ($aValue['public'] == 0) {
                        // category is locked
                        $aAnchorClass = ($noStart || $noOnline) ? 'on_error_locked' : 'on_locked';
                    } else {
                        // category is public
                        $aAnchorClass = ($noStart || $noOnline) ? 'on_error' : 'on';
                    }
                } else {
                    // category is offline
                    if ($aValue['public'] == 0) {
                        // category is locked
                        $aAnchorClass = ($noStart || $noOnline) ? 'off_error_locked' : 'off_locked';
                    } else {
                        // category is public
                        $aAnchorClass = ($noStart || $noOnline) ? 'off_error' : 'off';
                    }
                }

                if ($aValue['islast'] == 1) {
                    $aCssClasses[] = 'last';
                }
                if ($aValue['collapsed'] == 1 && is_array($this->_navigationTree[$currCatId])) {
                    $aCssClasses[] = 'collapsed';
                }
                if ($aValue['active']) {
                    $aCssClasses[] = 'active';
                }

                $cfgdata  = 'idcat' . $currCatId;
                $cssClass = (count($aCssClasses) > 0) ? ' class="'.implode(' ', $aCssClasses).'"' : '';

                // check level setting
                $currLevel    = (int) $aValue['level'];
                $currLevelStr = $currLevel;
                $newLevel     = $currLevel;
                if ($currLevel !== (int) $level) {
                    $newLevel     = $level;
                    $currLevelStr = "<span style='color:red;'>{$currLevel}</span> -&gt; <span style='color:green;'>$level</span>";
                }

                if ($prevLevel < $currLevel) {
                    // level up, should preid start with 0
                    $tmpPreId  = 0;
                    $prevLevel = $level;
                } elseif (isset($this->_catLevelsCache[$aValue['level']]) && count($this->_catLevelsCache[$aValue['level']]) > 2) {
                    // get pervious stored id fromlevel cache
                    $tmpPreId = $this->_catLevelsCache[$aValue['level']][count($this->_catLevelsCache[$aValue['level']])-2];
                } else {
                    $tmpPreId = $preIdCat;
                }

                // check preid setting
                $currPreId    = (int) $aValue['preid'];
                $currPreIdStr = $currPreId;
                $newPreId     = $currPreId;
                if ($currPreId != $tmpPreId) {
                    $newPreId     = $tmpPreId;
                    $currPreIdStr = "<span style='color:red;'>{$currPreId}</span> -&gt; <span style='color:green;'>$tmpPreId</span>";
                }

                // check postid setting
                $currPostId    = (int) $aValue['postid'];
                $currPostIdStr = $currPostId;
                $newPostId     = $currPostId;

                // check parentid setting
                $currParentId    = (int) $aValue['parentid'];
                $currParentIdStr = $currParentId;
                $newParentId     = $currParentId;

                // render cat item
                $tplCat->reset();
                $tplCat->set('s', 'ANCHOR_CLASS', $aAnchorClass);
                $tplCat->set('s', 'IDCAT', $currCatId);
                $tplCat->set('s', 'NAME', $name);
                $tplCat->set('s', 'CURR_LEVEL', $currLevel);
                $tplCat->set('s', 'NEW_LEVEL', $newLevel);
                $tplCat->set('s', 'CURR_PARENTID', $currParentId);
                $tplCat->set('s', 'NEW_PARENTID', $newParentId);
                $tplCat->set('s', 'CURR_PREID', $currPreId);
                $tplCat->set('s', 'NEW_PREID', $currPreId);
                $tplCat->set('s', 'CURR_POSTID', $currPostId);
                $tplCat->set('s', 'NEW_POSTID', $newPostId);
                $tplCat->set('s', 'CURR_LEVEL_STR', $currLevelStr);
                $tplCat->set('s', 'CURR_PARENTID_STR', $currParentId);
                $tplCat->set('s', 'CURR_PREID_STR', $currPreIdStr);
                $tplCat->set('s', 'CURR_POSTID_STR', $currPostIdStr);
                $mstr = $tplCat->generate(1);

                // Build Tree
                $tpl->set('d', 'CFGDATA',   $cfgdata);
                if (is_array($this->_navigationTree[$currCatId])) {
                    $tpl->set('d', 'SUBCATS',  $this->_processStrTree($currCatId, $level+1, $level));
                    $tpl->set('d', 'COLLAPSE', '<a href="#"> </a>');
                    $this->_wholeList[] = $currCatId;
                } else {
                    $tpl->set('d', 'SUBCATS',  '');
                    $tpl->set('d', 'COLLAPSE', '<span> </span>');
                }
                $tpl->set('d', 'CAT',       $mstr);
                $tpl->set('d', 'CSS_CLASS',	$cssClass);

                $tpl->next();

            } else {
                if (is_array($this->_navigationTree[$currCatId])) {
                    $sTpl = $this->_processStrTree($currCatId, $level+1, $level);
                    if (!preg_match('/^<ul>\s*<\/ul>$/', $sTpl)) {
                        $tpl->set('d', 'CFGDATA',   '0-0-0-0-0-0-0-0-0');
                        $tpl->set('d', 'SUBCATS',   $sTpl);
                        $tpl->set('d', 'COLLAPSE',  '<a href="#"></a>');
                        $tpl->set('d', 'CAT',       '<a class="off_disabled" href="#">' . $name . '</a>');
                        $tpl->set('d', 'CSS_CLASS',	' class="active"');
                        $tpl->next();
                    }
                    $this->_wholeList[] = $currCatId;
                }
            }

            $preIdCat = $currCatId;

        }
        return $tpl->generate(1);
    }


    /**
     * Initializes client and language, creates also the select options
     * @return  void
     */
    protected function _initClientAndLanguage()
    {
        $db = ConMp_Registry::get('oDB');

        // get available clients
        cInclude('classes', 'contenido/class.client.php');
        $clientColl = new cApiClientCollection();
        $clients = $clientColl->getAvailableClients();
        if (!is_array($clients)) {
            die('No available clients!');
        }

        // client select options
        foreach ($clients as $id => $item) {
            $sel = ($this->_client == $id) ? ' selected="selected"' : '';
            if ($sel !== '') {
                $this->_clientName = $item['name'];
            }
            $this->_changeClientOptions .= '<option value="' . $id . '"' . $sel . '>' . $item['name'] . '</option>';
        }

        // language select options for current client
        $sql = '
        SELECT l.idlang, l.name FROM ' . $this->_cfg->tab['clients'] . ' AS c, ' . $this->_cfg->tab['lang'] . ' AS l, ' . $this->_cfg->tab['clients_lang'] . ' AS cl
        WHERE c.idclient = ' . $this->_client . ' AND c.idclient = cl.idclient AND cl.idlang = l.idlang AND l.active = 1
        ORDER BY l.name
        ';
        $this->_db->query($sql);
        while ($this->_db->next_record()) {
            $sel = ($this->_lang == $this->_db->Record['idlang']) ? ' selected="selected"' : '';
            if ($sel !== '') {
                $this->_languageName = $this->_db->Record['name'];
            }
            $this->_changeLanguageOptions .= '<option value="' . $this->_db->Record['idlang'] . '"' . $sel . '>' . $this->_db->Record['name'] . '</option>';
        }
    }


    /**
     * Renders the navigation tree item. Fills the passed template object with
     * its data.
     *
     * Calls self recursively, if the navigation subtree item contains also
     * a subtree.
     *
     * @param   int             $iIdcat
     * @param   ConMp_Template  $tpl
     * @return  void
     */
    protected function _renderNavigationTreeItem($iIdcat, ConMp_Template $tpl)
    {
        $iIdcat = (int) $iIdcat;

        foreach ($this->_navigationTree[$iIdcat] as $sKey => $aValue) {
            $idcat    = (int) $aValue['idcat'];
            $name     = $aValue['name'];
            $level    = (int) $aValue['level'];
            $parentid = (int) $aValue['parentid'];
            $preid    = (int) $aValue['preid'];
            $postid   = (int) $aValue['postid'];

            if ($level > 0) {
                $nameAttrib = ' style="padding-left:' . (20 * $level) . 'px;"';
            } else {
                $nameAttrib = '';
            }

            $tpl->set('d', 'IDCAT', $idcat);
            $tpl->set('d', 'NAMEATTRIB', $nameAttrib);
            $tpl->set('d', 'NAME', $name);
            $tpl->set('d', 'LEVEL', $level);
            $tpl->set('d', 'PARENTID', $parentid);
            $tpl->set('d', 'PREID', $preid);
            $tpl->set('d', 'POSTID', $postid);
            $tpl->next();

            if (is_array($this->_navigationTree[$idcat])) {
                $this->_renderNavigationTreeItem($idcat, $tpl);
            }
        }
    }


    /**
     * Stores passed category id and its level
     * @param   int  $catid
     * @param   int  $level
     * @return  void
     */
    protected function _addToLevelCache($catid, $level)
    {
        // save category level
        if (!isset($this->_catLevelsCache[$level])) {
            $this->_catLevelsCache[$level] = array($catid);
        } elseif (!in_array($catid, $this->_catLevelsCache[$level])) {
            $this->_catLevelsCache[$level][] = $catid;
        }
    }
}
