<?php
/**
 * Contenido extension pack (mpContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Template
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2009-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


defined('CON_FRAMEWORK') or die('Illegal call');


/**
 * Template control element example. Returns a simple form.
 *
 * Usage:
 * Define a template control tag in your layout as follows:
 * [code]
 * <conControl:Example action="{CONTENIDOX_PAGE}" id="MyControl_id" labelUser="Benutzername" labelPassword="Sesam ffne dich" />
 * [/code]
 * - A template control tag must be a valid XML Element without a closing tag.
 * - Each template control tag starts with the pattern conControl:ControlName where
 *   conControl is the tag identifier and ControlName the name of the control.
 * - Existing attributes and their values will be passed to the control element.
 *   This means that they could be used to configure the control element.
 * - Each control element must have a corresponding file inside configured control
 *   folder, see $cfg['mpContenidoXApp']['controlsFolder'].
 *   The filename has to be "ControlName" + ".php", e. g. Example.php
 * - The control file must contain a class whose name consists of
 *   "TemplateControl_" + "ControlName", e. g. TemplateControl_Example
 * - Each control class must implement the abstract function process() which should
 *   return the output code for the defined control tag.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Template
 * @author      Murat Purc <murat@purc.de>
 */
class TemplateControl_Example extends ConMp_Template_Control_Abstract
{
    public function process() {
        $formAction    = $this->getParameter('action', CONTENIDOX_PAGE);
        $id            = $this->getParameter('id', 'frmId');
        $name          = $this->getParameter('name', 'frmName');
        $method        = ($this->getParameter('method') == 'GET') ? 'GET' : 'POST';
        $labelUser     = $this->getParameter('labelUser', 'User');
        $labelPasswort = $this->getParameter('labelPasswort', 'Password');
        $submitValue   = $this->getParameter('submitValue', 'Submit');

        return '
<form action="' . $formAction . '" id="' . $id . '" name="' . $name . '" method="' . $method . '">
    <label for="username">' . $labelUser . '</label><input type="text" id="username" name="username" value="" /><br />
    <label for="password">' . $labelPasswort . '</label><input type="password" id="password" name="password" value="" /><br />
    <br />
    <input type="submit" name="login" value="' . $submitValue . '" />
</form>
        ';

    }
}