<?php
/**
 * Contenido extension pack (mpContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Template
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2009-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


defined('CON_FRAMEWORK') or die('Illegal call');


/**
 * Template control element example. Returns a simple form.
 *
 * Usage:
 * Define a template control tag in your layout as follows:
 * [code]
 * <conControl:Loginform action="{CONTENIDOX_PAGE}" id="MyControl_id" labelUser="Benutzername" labelPassword="Sesam ffne dich" />
 * [/code]
 * - A template control tag must be a valid XML Element without a closing tag.
 * - Each template control tag starts with the pattern conControl:ControlName where
 *   conControl is the tag identifier and ControlName the name of the control.
 * - Existing attributes and their values will be passed to the control element.
 *   This means that they could be used to configure the control element.
 * - Each control element must have a corresponding file inside configured control
 *   folder, see $cfg['mpContenidoXApp']['controlsFolder'].
 *   The filename has to be "ControlName" + ".php", e. g. Loginform.php
 * - The control file must contain a class whose name consists of
 *   "TemplateControl_" + "ControlName", e. g. TemplateControl_Example
 * - Each control class must implement the abstract function process() which should
 *   return the output code for the defined control tag.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Template
 * @author      Murat Purc <murat@purc.de>
 */
class TemplateControl_Loginform extends ConMp_Template_Control_Abstract
{
    public function process()
    {
        $formAction    = $this->getParameter('action', CONTENIDOX_PAGE);
        $id            = $this->getParameter('id', 'frmId');
        $name          = $this->getParameter('name', 'frmName');
        $method        = ($this->getParameter('method') == 'GET') ? 'GET' : 'POST';
        $labelUser     = $this->getParameter('labelUser', 'User');
        $labelPassword = $this->getParameter('labelPassword', 'Password');
        $submitValue   = $this->getParameter('submitValue', 'Submit');

        $sess = ConMp_Registry::get('oSess');
        $area = 'templatecontrolexample';

        $request = ConMp_Request::getInstance();

        $msg  = '';
        $user = '';

        if ($request->isPost() && $request->getParam('login')) {
            $success = false;
            $user = $request->getParam('username');
            $pw   = $request->getParam('password');
            if ($user == 'admin' && $pw !== 'admin') {
                $msg = '<p style="color:red;">Wrong password</p>';
            } elseif ($user !== 'admin' && $pw == 'admin') {
                $msg = '<p style="color:red;">Wrong username</p>';
            } elseif ($user !== 'admin' && $pw !== 'admin') {
                $msg = '<p style="color:red;">Wrong username and password</p>';
            } else {
                $success = true;
                $msg = '<p style="color:green;">Welcome ' . $user . '</p>';
            }
        }

        $app = ConMp_Application_Backend::getInstance();
        $templateFile = $app->getTemplatePath() . $app->getTemplateFile();

        $tpl = ConMp_Template_Extended::getNewInstance($templateFile, 'PARTIAL_LOGIN_FORM');
        $tpl->set('s', 'FORM_ACTION', $formAction);
        $tpl->set('s', 'FORM_ID', $id);
        $tpl->set('s', 'FORM_NAME', $name);
        $tpl->set('s', 'FORM_METHOD', $method);
        $tpl->set('s', 'SESS_ID', $sess->id);
        $tpl->set('s', 'AREA', $app->getController()/*$area*/);
        $tpl->set('s', 'MSG', $msg);
        $tpl->set('s', 'LABEL_USER', $labelUser);
        $tpl->set('s', 'USERNAME', htmlspecialchars($user, ENT_QUOTES));
        $tpl->set('s', 'LABEL_PASSWORT', $labelPassword);
        $tpl->set('s', 'SUBMIT_VALUE', $submitValue);
        return $tpl->generate(1);
    }
}