<?php
/**
 * Contenido extension pack (mpContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  HelperTemplate
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2009-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


defined('CON_FRAMEWORK') or die('Illegal call');


/**
 * Template helper calculator.
 *
 * Usage:
 * Define a template helper replacement wildcard in your layout as follows:
 * [code]
 * {{Name%%Function%%Param1%%Param2...}}
 * [/code]
 * - A helper replacement wildcard has the format
 *       "{{" + "Helpername" + "%%" + "Function" + "%%" + "AdditionalParameter" + "}}"
 *   where "Helpername" is the name of helper, "Function" the function to invoke and
 *   "AdditionalParameter" the parameter to pass to the function
 *   Example: {{Calculator%%add%%2%%3}}
 * - The filename for the helper must be available in configured templates helper folder,
 *   see $cfg['mpContenidoXApp']['helperTemplatesFolder'].
 *   The filename has to be "Helpername" + ".php", e. g. Calculator.php
 * - Helper file must contain a class whose name consists of
 *   "HelperTemplate_" + "Helpername", e. g. HelperTemplate_Calculator
 * - The function of the helper should return the value which wil be replace
 *   the replacement wildcard.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  HelperTemplate
 * @author      Murat Purc <murat@purc.de>
 */
class HelperTemplate_Calculator {

    public function add($number1, $number2)
    {
        return (float) $number1 + (float) $number2;
    }

    public function subtract($number1, $number2)
    {
        return (float) $number1 - (float) $number2;
    }

    public function divide($number1, $number2)
    {
        if ((float) $number2 == 0) {
            throw new Exception('Divisor is 0, division by zero not possible');
        }
        return (float) $number1 / (float) $number2;
    }

    public function multiply($number1, $number2)
    {
        return (float) $number1 * (float) $number2;
    }
}