<?php
/**
 * Project:
 * Contenido Content Management System
 *
 * Description:
 * An example for template helper.
 *
 * Usage:
 * Define a template helper replacement wildcard in your layout as follows:
 * [code]
 * {{Name%%Function}}
 * [/code]
 * - A helper replacement wildcard has the format
 *       "{{" + "Helpername" + "%%" + "Function" + "}}"
 *   where "Helpername" is the name of helper and "Function" the function to invoke
 *   Example: {{Debug%%flush}}
 * - The filename for the helper must be available in configured templates helper folder,
 *   see $cfg['mpContenidoXApp']['helperTemplatesFolder'].
 *   The filename has to be "Helpername" + ".php", e. g. Debug.php
 * - Helper file must contain a class whose name consists of
 *   "HelperTemplate_" + "Helpername", e. g. HelperTemplate_Debug
 * - The function of the helper should return the value which wil be replace
 *   the replacement wildcard.
 *
 * Requirements:
 * @con_php_req 5.0
 *
 * @package    Contenido Backend
 * @version    0.1
 * @author     Murat Purc
 * @copyright  Murat Purc <www.purc.de>
 * @license    http://www.contenido.org/license/LIZENZ.txt
 *
 * {@internal
 *   created 2009-12-18 Murat Purc
 *   $Id$
 * }}
 */


/**
 * Template helper debug.
 *
 * @author      Murat Purc <murat@purc.de>
 * @copyright 	2009 Murat Purc
 * @category    ContenidoX
 * @package     HelperTemplate
 * @subpackage  Debug
 */
class HelperTemplate_Debug {

    /**
     * Returns the collected debug informations. Adds also the collected
     * SQL statements, if available, to the debugger.
     *
     * @return  string
     */
    public function flush()
    {
        if (isset($GLOBALS['DB_Contenido_SQL']) && is_array($GLOBALS['DB_Contenido_SQL'])) {
            conX_debugAdd(count($GLOBALS['DB_Contenido_SQL']['default']), 'count sql');
            $sqls = '';
            foreach ($GLOBALS['DB_Contenido_SQL']['default'] as $p => $item) {
                $sql = $item['sql'];
                $sql = str_replace("\r\n", "\n", $sql); // replace win -> unix
                $sql = str_replace("\r", "\n", $sql);   // replace max -> unix
                $sql = preg_replace("/[\n\n]+/", "\n", $sql); // replace multiple line feeds
                $sql = preg_replace("/[\s\s]+/", " ", $sql); // replace white spaces
#                $sqls .= trim($sql) . "\n" . $item['time'] . "\n\n";
#                if (strpos(strtolower($sql), 'select') === false) {
                    $sqls .= trim($sql) . "\n\n";
#                }
            }
            conX_debugAdd($sqls, 'all sql');
        }

        return '<div>' . conX_debugGetAll() . '</div>';
    }

}