<?php
/**
 * Contenido extension pack (mpContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Application
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2009-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


defined('CON_FRAMEWORK') or die('Illegal call');


/**
 * ContenidoX backend application.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Application
 * @author      Murat Purc <murat@purc.de>
 */
class ConMp_Application_Backend
{
    private static $_instance;

    private static $_run = false;

    private $_module;

    private $_modulePath;

    private $_controller;

    private $_controllerPath;

    private $_action;

    private $_templateFile;

    private $_templatePath;


    /**
     * Constructor, set some properties
     * @return  void
     */
    protected function __construct()
    {
        $request = ConMp_Request::getInstance();
        $config  = ConMp_Config::getInstance();

        $module = $config->mpContenidoXApp['module'];
        $this->setModule($module);
        $this->setModulePath(CONMP_APPLICATION_PATH . (($module == 'default') ? '' : $module . '/'));

        $controller = ($request->getParam('area')) ? trim(strip_tags($request->getParam('area'))) : $config->mpContenidoXApp['defaultController'];
        $this->setController($controller);

        $this->setControllerPath($this->getModulePath() . $config->mpContenidoXApp['controllerFolder'] . '/');

        $this->setTemplateFile($controller . '.html');
        $this->setTemplatePath($this->getModulePath() . $config->mpContenidoXApp['templatesFolder'] . '/');
    }


    /**
     * Disable cloning
     */
    private function __clone()
    {
    }


    /**
     * Returns self instance
     *
     * @return  ContenidoX_Page
     */
    public static function getInstance()
    {
        if (self::$_instance == null) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }


    /**
     * Main method to invoke or backend initialization.
     * @return  void
     */
    public function run()
    {
        if (self::$_run) {
            return;
        }
        self::$_run = true;

        return $this->_run();
    }


    public function setModule($module)
    {
        $this->_module = $module;
    }

    public function getModule()
    {
        return $this->_module;
    }


    public function setModulePath($modulePath)
    {
        $this->_modulePath = $modulePath;
    }

    public function getModulePath()
    {
        return $this->_modulePath;
    }


    public function setController($controller)
    {
        $this->_controller = $controller;
    }

    public function getController()
    {
        return $this->_controller;
    }

    public function setControllerPath($controllerPath)
    {
        $this->_controllerPath = $controllerPath;
    }

    public function getControllerPath()
    {
        return $this->_controllerPath;
    }

    public function setAction($action)
    {
        $this->_action = $action;
    }

    public function getAction()
    {
        return $this->_action;
    }

    public function setTemplateFile($templateFile)
    {
        $this->_templateFile = $templateFile;
    }

    public function getTemplateFile()
    {
        return $this->_templateFile;
    }

    public function setTemplatePath($templatePath)
    {
        $this->_templatePath = $templatePath;
    }

    public function getTemplatePath()
    {
        return $this->_templatePath;
    }

    protected function _run()
    {
        $request = ConMp_Request::getInstance();
        $config  = ConMp_Config::getInstance();

        $module             = $this->getModule();
        $modulePath         = $this->getModulePath();
        $controller         = $this->getController();
        $className          = ucfirst($controller) . 'Controller';
        $file               = $this->getControllerPath() . $className . '.php';
        $action             = $request->getParam('action', 'index');
        $actionFunction     = $action . 'Action';
        $preDispatchStatus  = null;
        $mainDispatchStatus = null;
        $postDispatchStatus = null;

        // require and instantiate controller
        require_once($file);
        $instance = new $className();
        $instance->setModulePath($modulePath);
        $instance->setTemplateFile($this->getTemplatePath() . $this->getTemplateFile());

        // validate action function
        $actionIsCallable = is_callable(array($instance, $actionFunction));
        if (!$actionIsCallable) {
            throw new Exception($className . '->' . $actionFunction . ' is not callable');
        }

        // invoke predispatch
        $preDispatchStatus = $instance->preDispatch();

        // invoke action
        if ($preDispatchStatus) {
            $res = call_user_func_array(
                array($instance, $actionFunction), array()
            );
            $mainDispatchStatus = (is_bool($res)) ? $res : true;
        }

        // invoke postdispatch
        if ($mainDispatchStatus) {
            $postDispatchStatus = $instance->postDispatch();
        }

        if (!$preDispatchStatus || !$mainDispatchStatus || !$postDispatchStatus) {
            throw new Exception($className . '->' . $actionFunction . ': Hmm, something went wrong');
        }

        return true;
    }

}