<?php
/**
 * Contenido extension pack (mpContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Controller
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2009-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


defined('CON_FRAMEWORK') or die('Illegal call');


/**
 * Abstract Contenido backend controller class.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Controller
 * @author      Murat Purc <murat@purc.de>
 */
abstract class ConMp_Controller_Backend_Abstract
{
    protected $_cfg;
    protected $_client;
    protected $_lang;
    protected $_area;
    protected $_action;
    protected $_sess;
    protected $_auth;
    protected $_perm;
    protected $_request;
    protected $_modulePath;
    protected $_templateFile;


    public function __construct() {
        $this->_cfg     = ConMp_Config::getInstance();
        $this->_client  = (int) $GLOBALS['client'];
        $this->_lang    = (int) $GLOBALS['lang'];
        $this->_area    = $GLOBALS['area'];
        $this->_action  = $GLOBALS['action'];
        $this->_sess    = ConMp_Registry::get('oSess');
        $this->_auth    = ConMp_Registry::get('oAuth');
        $this->_perm    = ConMp_Registry::get('oPerm');
        $this->_request = ConMp_Registry::get('oRequest');
        $this->_templateFile = '';
#        if (!$this->_perm->have_perm_area_action($this->_area)) {
#            $GLOBALS['notification']->displayNotification('error', i18n('Permission denied'));
#        }
#        if ($this->_action && !$this->_perm->have_perm_area_action($this->_area, $this->_action)) {
#            $GLOBALS['notification']->displayNotification('error', i18n('Permission denied'));
#        }
        $this->init();
    }


    public function init() {
        return true;
    }


    /**
     * Pre dispatch method, will be called before the action.
     * Childs can overwrite this method.
     *
     * @return  bool  If return is true the action will be invoked. A false return value aborts the
     *                process, the action method will not invoked.
     */
    public function preDispatch() {
        return true;
    }


    /**
     * Post dispatch method, will be called after the action.
     * Childs can overwrite this method.
     *
     * @return  bool
     */
    public function postDispatch() {
        return true;
    }


    public function setTemplateFile($templateFile) {
        if (!is_file($templateFile)) {
            throw new Exception("Invalid template file $templateFile");
        }
        $this->_templateFile = $templateFile;
    }


    public function setModulePath($modulePath) {
        if (!is_dir($modulePath)) {
            throw new Exception("Invalid module path $modulePath");
        }
        $this->_modulePath = $modulePath;
    }
}
