<?php
/**
 * Contenido extension pack (mpContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Template
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2009-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


defined('CON_FRAMEWORK') or die('Illegal call');


/**
 * Template syntax control tag.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Template
 * @author      Murat Purc <murat@purc.de>
 */
class ConMp_Template_Compiler_ControlItem {
    private $_name = '';
    private $_tag = '';
    private $_attributes = array();
    private $_cfg;
    private $_path = '';

    public function __construct($name, $tag)
    {
        $this->_name = $name;
        $this->_tag  = $tag;
        $this->_cfg  = & $GLOBALS['cfg'];
        $app = ConMp_Application_Backend::getInstance();
        $this->_path = $app->getModulePath() . $this->_cfg['mpContenidoXApp']['controlsFolder'] . '/';

    }

    public function getName()
    {
        return $this->_name;
    }

    public function getTag()
    {
        return $this->_tag;
    }

    public function setAttribute($name, $value)
    {
        $this->_attributes[$name] = $this->_parseAttributeValue($value);
    }

    public function getAttribute($name)
    {
        return ($this->hasAttribute($name)) ? $this->_attributes[$name] : null;
    }

    public function hasAttribute($name)
    {
        return isset($this->_attributes[$name]);
    }

    public function processControl()
    {
        try {
            $file = $this->_getControlFile();
            require_once($file);

            $class    = $this->_getControlClass();
            $function = $this->_getControlClassFunction();

            $obj = new $class();
            $call = array($obj, $function);
            if (is_callable($call)) {
                foreach ($this->_attributes as $name => $value) {
                    $obj->setParameter($name, $value);
                }
                return call_user_func($call);
            } else {
                return 'Not callable ' . $class . '->' . $function;
            }
        } catch (Exception $e) {
            // @todo: error handling
            return $e->getMessage();
        }
        return '';
    }

    private function _parseAttributeValue($value)
    {
        $value = ConMp_Template_Compiler_Function::getInstance()->process($value);

        foreach ($this->_cfg['mpContenidoXApp']['templateReplacements'] as $search => $replace) {
            $value = str_replace($search, $replace, $value);
        }
        return $value;
    }

    private function _getControlFile()
    {
        if ($this->hasAttribute('conFile')) {
            $file = $this->getAttribute('conFile');
        } else {
            $file = $this->_path . ucfirst($this->getName()) . '.php';
        }
        if (!is_file($file)) {
            throw new Exception('Could not find control tag file ' . $file);
        }
        return $file;
    }

    private function _getControlClass()
    {
        if ($this->hasAttribute('conClass')) {
            $class = $this->getAttribute('conClass');
        } else {
            $class = 'TemplateControl_' . $this->getName();
        }
        if (!class_exists($class)) {
            throw new Exception('Could not find control tag class');
        }
        return $class;
    }

    private function _getControlClassFunction()
    {
        if ($this->hasAttribute('conFunction')) {
            $function = $this->getAttribute('conFunction');
        } else {
            $function = 'process';
        }
        return $function;
    }

}
