<?php
/**
 * Contenido extension pack (mpContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Template
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2009-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


defined('CON_FRAMEWORK') or die('Illegal call');


/**
 * Template compiler function item for template functions
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Template
 * @author      Murat Purc <murat@purc.de>
 */
class ConMp_Template_Compiler_FunctionItem
{
    private static $_instance = null;
    private static $_functionInstances = array();
    private $_path;
    private $_tag       = '';
    private $_class     = '';
    private $_function  = '';
    private $_parameter = array();


    protected function __construct()
    {
        $app = ConMp_Application_Backend::getInstance();
        $this->_path = $app->getModulePath() . $GLOBALS['cfg']['mpContenidoXApp']['helperTemplatesFolder'] . '/';
    }


    private function __clone()
    {
    }


    public static function getInstance()
    {
        if (!isset(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }


    public function setClass($class)
    {
        $this->_class = 'HelperTemplate_' . $class;
        $this->_loadClass($class);
    }


    public function setTag($tag)
    {
        $this->_tag = $tag;
    }


    public function getTag()
    {
        return $this->_tag;
    }


    public function setFunction($function)
    {
        $this->_function = $function;
    }


    public function getFunction()
    {
        return $this->_function;
    }


    public function setParameter(array $parameter)
    {
        $this->_parameter = $parameter;
    }


    public function getParameter()
    {
        return $this->_parameter;
    }


    public function process($template)
    {
        if (empty($this->_class)) {
            throw new Exception('No function class initialized');
        } elseif (empty($this->_tag)) {
            throw new Exception('No tag to process for class ' . $this->_class);
        } elseif (empty($this->_function)) {
            throw new Exception('No function to process for class ' . $this->_class);
        }

        $call = array(self::$_functionInstances[$this->_class], $this->_function);
        if (is_callable($call)) {
            try {
                $replacement = call_user_func_array($call, $this->_parameter);
            } catch (Exception $e) {
                $replacement = $e->getMessage();
            }
        } else {
            $replacement = 'Not callable.' . $this->_tag;
        }
        return str_replace($this->_tag, $replacement, $template);
    }


    protected function _loadClass($class)
    {
        if (!isset(self::$_functionInstances[$this->_class])) {
            $file = $this->_path . '/' . $class . '.php';
            if (!is_file($file)) {
                throw new Exception('Could not find file for template function ' . $class . ' in ' . $file);
            }
            require_once($file);
            if (!class_exists($this->_class)) {
                throw new Exception('Could not find class for template function ' . $class);
            }
            self::$_functionInstances[$this->_class] = new $this->_class();
        }
    }
}