<?php
/**
 * Contenido extension pack (mpContenidoX), consists of multiple classes to
 * provide more features for Plugins/Modules.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Template
 * @author      Murat Purc <murat@purc.de>
 * @copyright   Copyright (c) 2009-2010 Murat Purc (http://www.purc.de)
 * @license     http://www.gnu.org/licenses/gpl-2.0.html - GNU General Public License, version 2
 * @version     $Id$
 */


defined('CON_FRAMEWORK') or die('Illegal call');


/**
 * Extended template class.
 *
 * @package     Plugin_mpContenidoX
 * @subpackage  Template
 * @author      Murat Purc <murat@purc.de>
 */
class ConMp_Template_Extended extends ConMp_Template
{
    const WHOLE_TEMPLATE = 'whole_tpl';

    /**
     * Multiple instances
     *
     * @var  ConMp_Template_Extended
     */
    private static $_instances = null;

    /**
     * Whole content of template file
     *
     * @var  string
     */
    private static $_tplContent = null;

    /**
     * Extracted template part from template content (@see ConMp_Template_Extended::$_tplContent)
     *
     * @var  string
     */
    protected $_tplPart;

    private $_helperInstances = array();


    protected function __construct($template, $token)
    {
        if (self::$_tplContent == null) {
            if (is_file($template)) {
                self::$_tplContent = file_get_contents($template);
            } else {
                self::$_tplContent = $template;
            }
        }
        if ($token == self::WHOLE_TEMPLATE) {
            $this->_tplPart = self::$_tplContent;
        } else {
            $this->_tplPart = $this->_extractTemplatePart($token);
        }
    }


    private function __clone()
    {
        // donut
    }


    public static function getInstance($template, $token=ConMp_Template_Extended::WHOLE_TEMPLATE)
    {
        $key = md5($template . $token);
        if (self::$_instances == null || !isset(self::$_instances[$key])) {
            self::$_instances[$key] = new self($template, $token);
        }
        return self::$_instances[$key];
    }


    public static function getNewInstance($template, $token=ConMp_Template_Extended::WHOLE_TEMPLATE)
    {
        return new self($template, $token);
    }


    public function getTplPart()
    {
        return $this->_tplPart;
    }


    public function generate($return=0, $note=0)
    {
        if ($this->_tplPart) {
            $template = $this->_tplPart;
            $controlTagProc = new ConMp_Template_Compiler_Control();
            $template = $controlTagProc->process($template);
            $template = ConMp_Template_Compiler_Function::getInstance()->process($template);
            if ($return) {
                return parent::generate($template, $return, $note);
            } else {
                parent::generate($template, $return, $note);
            }
        }
    }


    /**
     * Extracts the template part from the template content
     *
     * @param    string  $name  The name of the part to extract
     * @return   string  The extracted template part
     */
    protected function _extractTemplatePart($name)
    {
        $startToken = '<!-- #####' . $name . '##### -->';
        $endToken   = '<!-- #####/' . $name . '##### -->';

        $startTokenRegEx = preg_quote($startToken, '/');
        $endTokenRegEx   = preg_quote($endToken, '/');

        $pattern = '/^(.*)' . $startTokenRegEx . '(.*?)' . $endTokenRegEx . '(.*)$/s';

        if (preg_match($pattern, self::$_tplContent, $matches)) {
            return trim($matches[2]);
        } else {
            return 'Error: ' . $name;
        }
    }
}
