<?php
/**
 * Project:
 * Contenido Content Management System
 *
 * Description:
 * ContenidoX main file, based on contenido main.php
 *
 * Requirements:
 * @con_php_req 5.0
 *
 *
 * @package    ContenidoX Backend
 * @version    0.1
 * @author     Olaf Niemann, Jan Lengowski
 * @author     Murat Purc
 * @copyright  four for business AG <www.4fb.de>
 * @license    http://www.contenido.org/license/LIZENZ.txt
 * @link       http://www.4fb.de
 * @link       http://www.contenido.org
 *
 * {@internal
 *   created 2009-11-20 Murat Purc
 *   $Id$
 * }}
 *
 */


defined('CON_FRAMEWORK') or define('CON_FRAMEWORK', true);

// include security class and check request variables
include_once('./classes/class.security.php');
Contenido_Security::checkRequests();

if (isset($_REQUEST['changeclient'])) {
    $_REQUEST['changeclient'] = intval($_REQUEST['changeclient']);
    $changeclient = intval($_REQUEST['changeclient']);
}
if (isset($_REQUEST['changelang'])) {
    $_REQUEST['changelang'] = intval($_REQUEST['changelang']);
    $changelang = intval($_REQUEST['changelang']);
}

include_once('./includes/startup.php');
cInclude('includes', 'functions.general.php');

$cfg['debug']['rendering'] = true;

$cfg['debug']['backend_exectime']['fullstart'] = getmicrotime();

cInclude('includes', 'functions.i18n.php');
cInclude('includes', 'functions.api.php');
cInclude('includes', 'functions.general.php');
cInclude('includes', 'functions.forms.php');

cInclude('classes', 'class.xml.php');
cInclude('classes', 'class.navigation.php');
cInclude('classes', 'class.template.php');
cInclude('classes', 'class.backend.php');
cInclude('classes', 'class.notification.php');
cInclude('classes', 'class.area.php');
cInclude('classes', 'class.action.php');
cInclude('classes', 'class.layout.php');
cInclude('classes', 'class.treeitem.php');
cInclude('classes', 'class.user.php');
cInclude('classes', 'class.group.php');
cInclude('classes', 'class.cat.php');
cInclude('classes', 'class.client.php');
cInclude('classes', 'class.inuse.php');
cInclude('classes', 'class.table.php');

page_open(array(
    'sess' => 'Contenido_Session',
    'auth' => 'Contenido_Challenge_Crypt_Auth',
    'perm' => 'Contenido_Perm'
));

i18nInit($cfg['path']['contenido'] . $cfg['path']['locale'], $belang);

require_once $cfg['path']['contenido'] . $cfg['path']['includes'] . 'functions.includePluginConf.php';

cInclude('includes', 'cfg_language_de.inc.php');


################################################################################

// Create Contenido classes
$db           = new DB_Contenido();
$notification = new Contenido_Notification();
$classarea    = new Area();
$classlayout  = new Layout();
$classclient  = new Client();
$classuser    = new User();
$currentuser  = new User();
$currentuser->loadUserByUserID($auth->auth['uid']);
$xml        = new XML_doc();
$tpl        = new Template();


// change client
if (isset($changeclient) && is_numeric($changeclient)) {
    $client = $changeclient;
    unset($lang);
}

// change language
if (isset($changelang) && is_numeric($changelang)) {
    unset($area_rights);
    unset($item_rights);
    $lang = $changelang;
}

// set client
if (!is_numeric($client) || (!$perm->have_perm_client('client['.$client.']') && !$perm->have_perm_client('admin['.$client.']'))) {
     // use first client which is accessible
    $sess->register('client');
    $sql = 'SELECT idclient FROM '.$cfg['tab']['clients'].' ORDER BY idclient ASC';
    $db->query($sql);
    while ($db->next_record()) {
        $mclient = $db->f('idclient');
        if ($perm->have_perm_client('client['.$mclient.']') || $perm->have_perm_client('admin['.$mclient.']') ) {
            unset($lang);
            $client = $mclient;
            break;
        }
    }
} else {
    $sess->register('client');
}

// set language
if (!is_numeric($lang) || $lang == '') {
    $sess->register('lang');
    //search for the first language of this client
    $sql = 'SELECT * FROM '.$cfg['tab']['lang'].' AS A, '.$cfg['tab']['clients_lang'].' AS B WHERE A.idlang=B.idlang AND idclient="'.(int) $client .'" ORDER BY A.idlang ASC';
    $db->query($sql);
    $db->next_record();
    $lang = $db->f('idlang');
} else {
    $sess->register('lang');
}

// send right encoding http header
sendEncodingHeader($db, $cfg, $lang);

$perm->load_permissions();

$sess->register('cfgClient');
$sess->register('errsite_idcat');
$sess->register('errsite_idart');

if ($cfgClient['set'] != 'set') {
     rereadClients();
}

// Initialize Contenido_Backend. Load all actions from the DB and check if permission is granted.
if ($cfg['debug']['rendering'] == true) {
    if (function_exists('memory_get_usage')) {
        $oldmemusage = memory_get_usage();
    }
}

$cfg['debug']['backend_exectime']['start'] = getmicrotime();


################################################################################
######## Additional initialization

cInclude('classes', 'Debug/Debug_VisibleAdv.class.php');

// this script name
define('CONTENIDOX_PAGE', basename(__FILE__));

//
require_once(CONMP_APPLICATION_PATH . 'Bootstrap.php');

ConMp_Bootstrap::getInstance()->run();

$request = ConMp_Registry::get('oRequest');

#$GLOBALS['cfg']['mpContenidoXApp']['contentDebug'] = ($request->getParam('debug') == '1');
$config = ConMp_Config::getInstance();
$config->mpContenidoXApp['contentDebug'] = ($request->getParam('debug') == '1');
#echo '<pre>';var_dump($config);echo '</pre>';

$application = ConMp_Application_Backend::getInstance();
ConMp_Registry::set('oApp', $application);

$result = $application->run();

################################################################################


$cfg['debug']['backend_exectime']['end'] = getmicrotime();

if ($cfg['debug']['rendering'] == true) {
    $timeController = sprintf('%0.5f', $cfg['debug']['backend_exectime']['end'] - $cfg['debug']['backend_exectime']['start']);
    $timeTotal      = sprintf('%0.5f', $cfg['debug']['backend_exectime']['end'] - $cfg['debug']['backend_exectime']['fullstart']);
    $msg  = '<br /><br />';
    $msg .= 'Rendering this controller (excluding contenido includes) took: ' . $timeController . ' sec.<br />';
    $msg .= 'Building the complete page took: ' . $timeTotal . ' sec.<br />';
    if (function_exists('memory_get_usage')) {
        $msg .= 'Include memory usage: '.human_readable_size(memory_get_usage()-$oldmemusage).'<br>';
        $msg .= 'Complete memory usage: '.human_readable_size(memory_get_usage()).'<br>';
    }
    echo $msg;
}

page_close();
